@@ -1,7 +1,6 @@
 use crate::event::{ClickState, EventPayload, EventProxy, RioEvent, RioEventType};
 use crate::ime::Preedit;
-use crate::router::{RouteWindow, Router};
-use crate::routes::RoutePath;
+use crate::router::{routes::RoutePath, Router};
 use crate::scheduler::{Scheduler, TimerId, Topic};
 use crate::screen::touch::on_touch;
 use crate::watcher::configuration_file_updates;
@@ -24,19 +23,19 @@ use rio_window::window::{CursorIcon, Fullscreen};
 use std::error::Error;
 use std::time::{Duration, Instant};
 
-pub struct Application {
+pub struct Application<'a> {
     config: rio_backend::config::Config,
     event_proxy: EventProxy,
-    router: Router,
+    router: Router<'a>,
     scheduler: Scheduler,
 }
 
-impl Application {
-    pub fn new(
+impl Application<'_> {
+    pub fn new<'app>(
         config: rio_backend::config::Config,
         config_error: Option<rio_backend::config::ConfigError>,
         event_loop: &EventLoop<EventPayload>,
-    ) -> Application {
+    ) -> Application<'app> {
         // SAFETY: Since this takes a pointer to the winit event loop, it MUST be dropped first,
         // which is done in `loop_exiting`.
         let clipboard =
@@ -95,7 +94,7 @@ impl Application {
     }
 }
 
-impl ApplicationHandler<EventPayload> for Application {
+impl ApplicationHandler<EventPayload> for Application<'_> {
     fn resumed(&mut self, _active_event_loop: &ActiveEventLoop) {
         #[cfg(not(any(target_os = "macos", windows)))]
         {
@@ -116,17 +115,12 @@ impl ApplicationHandler<EventPayload> for Application {
             return;
         }
 
-        let mut window = RouteWindow::new(
+        self.router.create_window(
             event_loop,
-            &self.event_proxy,
+            self.event_proxy.clone(),
             &self.config,
-            &self.router.font_library,
             None,
-            &self.router.clipboard,
-        )
-        .unwrap();
-        window.is_focused = true;
-        self.router.create_route_from_window(window);
+        );
 
         tracing::info!("Initialisation complete");
     }
@@ -185,11 +179,11 @@ impl ApplicationHandler<EventPayload> for Application {
                     drop(terminal);
                     if let Some(graphic_queues) = graphics {
                         for graphic_data in graphic_queues.pending {
-                            route.window.screen.sugarloaf.add_graphic(graphic_data);
+                            route.window.screen.sugarloaf.graphics.insert(graphic_data);
                         }
 
                         for graphic_data in graphic_queues.remove_queue {
-                            route.window.screen.sugarloaf.remove_graphic(&graphic_data);
+                            route.window.screen.sugarloaf.graphics.remove(&graphic_data);
                         }
                     }
                 }
@@ -232,6 +226,7 @@ impl ApplicationHandler<EventPayload> for Application {
                         }
                     }
 
+                    route.window.configure_window(&self.config);
                     route.update_config(&self.config, &self.router.font_library);
 
                     if let Some(error) = &config_error {
@@ -446,7 +441,7 @@ impl ApplicationHandler<EventPayload> for Application {
                         event_loop,
                         self.event_proxy.clone(),
                         &self.config,
-                        Some(route.window.winit_window.tabbing_identifier()),
+                        Some(&route.window.winit_window.tabbing_identifier()),
                         None,
                     );
 
@@ -577,7 +572,7 @@ impl ApplicationHandler<EventPayload> for Application {
                     active_event_loop,
                     self.event_proxy.clone(),
                     config,
-                    tab_id.clone(),
+                    tab_id.as_deref(),
                     Some(url),
                 );
             }
