@@ -177,7 +177,7 @@ impl Screen<'_> {
             }
         };
 
-        let renderer = Renderer::new(config, theme);
+        let renderer = Renderer::new(config, theme, font_library);
 
         let bindings = crate::bindings::default_key_bindings(
             config.bindings.keys.to_owned(),
@@ -219,7 +219,12 @@ impl Screen<'_> {
             sugarloaf_errors,
         )?;
 
-        sugarloaf.set_background_color(renderer.dynamic_background.1);
+        if cfg!(target_os = "macos") {
+            sugarloaf.set_background_color(None);
+        } else {
+            sugarloaf.set_background_color(Some(renderer.dynamic_background.1));
+        }
+
         if let Some(image) = &config.window.background_image {
             sugarloaf.set_background_image(image);
         }
@@ -323,7 +328,7 @@ impl Screen<'_> {
         );
 
         self.sugarloaf.layout_mut().update();
-        self.renderer = Renderer::new(config, current_theme);
+        self.renderer = Renderer::new(config, current_theme, font_library);
 
         for context in self.ctx().contexts() {
             let mut terminal = context.terminal.lock();
@@ -337,8 +342,13 @@ impl Screen<'_> {
         self.mouse
             .set_multiplier_and_divider(config.scroll.multiplier, config.scroll.divider);
 
-        self.sugarloaf
-            .set_background_color(self.renderer.dynamic_background.1);
+        if cfg!(target_os = "macos") {
+            self.sugarloaf.set_background_color(None);
+        } else {
+            self.sugarloaf
+                .set_background_color(Some(self.renderer.dynamic_background.1));
+        }
+
         if let Some(image) = &config.window.background_image {
             self.sugarloaf.set_background_image(image);
         }
@@ -490,7 +500,6 @@ impl Screen<'_> {
                     .into(),
             };
 
-            self.sugarloaf.mark_dirty();
             self.ctx_mut().current_mut().messenger.send_write(bytes);
 
             return;
@@ -558,7 +567,6 @@ impl Screen<'_> {
         };
 
         if !bytes.is_empty() {
-            self.sugarloaf.mark_dirty();
             self.scroll_bottom_when_cursor_not_visible();
             self.clear_selection();
 
@@ -1436,7 +1444,6 @@ impl Screen<'_> {
         // Enable IME so we can input into the search bar with it if we were in Vi mode.
         // self.window().set_ime_allowed(true);
 
-        self.sugarloaf.mark_dirty();
         self.render();
     }
 
@@ -1481,7 +1488,6 @@ impl Screen<'_> {
         // let vi_mode = self.get_mode().contains(Mode::VI);
         // self.window().set_ime_allowed(!vi_mode);
 
-        self.sugarloaf.mark_dirty();
         self.search_state.history_index = None;
 
         // Clear focused match.
@@ -1541,8 +1547,6 @@ impl Screen<'_> {
             // Update search highlighting.
             self.goto_match(MAX_SEARCH_WHILE_TYPING);
         }
-
-        self.sugarloaf.mark_dirty();
     }
 
     /// Reset terminal to the state before search was started.
@@ -1571,7 +1575,6 @@ impl Screen<'_> {
             drop(terminal);
         }
         self.search_state.display_offset_delta = 0;
-        self.sugarloaf.mark_dirty();
     }
 
     /// Jump to the first regex match from the search origin.
@@ -1640,8 +1643,6 @@ impl Screen<'_> {
         if should_reset_search_state {
             self.search_reset_state();
         }
-
-        self.sugarloaf.mark_dirty();
     }
 
     fn sgr_mouse_report(&mut self, pos: Pos, button: u8, state: ElementState) {
@@ -1877,21 +1878,24 @@ impl Screen<'_> {
         }
     }
 
-    pub fn render_assistant(&mut self, assistant: &crate::routes::assistant::Assistant) {
+    pub fn render_assistant(
+        &mut self,
+        assistant: &crate::router::routes::assistant::Assistant,
+    ) {
         self.sugarloaf.clear();
-        crate::routes::assistant::screen(&mut self.sugarloaf, assistant);
+        crate::router::routes::assistant::screen(&mut self.sugarloaf, assistant);
         self.sugarloaf.render();
     }
 
     pub fn render_welcome(&mut self) {
         self.sugarloaf.clear();
-        crate::routes::welcome::screen(&mut self.sugarloaf);
+        crate::router::routes::welcome::screen(&mut self.sugarloaf);
         self.sugarloaf.render();
     }
 
     pub fn render_dialog(&mut self, content: &str) {
         self.sugarloaf.clear();
-        crate::routes::dialog::screen(&mut self.sugarloaf, content);
+        crate::router::routes::dialog::screen(&mut self.sugarloaf, content);
         self.sugarloaf.render();
     }
 
