@@ -57,15 +57,15 @@ pub struct Developer {
     pub enable_fps_counter: bool,
     #[serde(default = "default_log_level", rename = "log-level")]
     pub log_level: String,
-    #[serde(rename = "log-file", default)]
-    pub log_file: bool,
+    #[serde(rename = "enable-log-file", default)]
+    pub enable_log_file: bool,
 }
 
 impl Default for Developer {
     fn default() -> Developer {
         Developer {
             log_level: default_log_level(),
-            log_file: false,
+            enable_log_file: false,
             enable_fps_counter: false,
         }
     }
@@ -255,18 +255,16 @@ impl Config {
 
                         if let Ok(light_loaded_theme) = Config::load_theme(&path) {
                             adaptive_colors.light = Some(light_loaded_theme.colors);
-                            println!("carregou");
                         } else {
-                            println!("failed to load light theme: {}", light_theme);
+                            warn!("failed to load light theme: {}", light_theme);
                         }
 
                         let dark_theme = &adaptive_theme.dark;
                         let path = tmp.join(dark_theme).with_extension("toml");
                         if let Ok(dark_loaded_theme) = Config::load_theme(&path) {
                             adaptive_colors.dark = Some(dark_loaded_theme.colors);
-                            println!("carregou");
                         } else {
-                            println!("failed to load dark theme: {}", dark_theme);
+                            warn!("failed to load dark theme: {}", dark_theme);
                         }
 
                         if adaptive_colors.light.is_some()
