@@ -119,6 +119,7 @@ pub(crate) struct State {
     standard_frame: Cell<Option<NSRect>>,
     is_simple_fullscreen: Cell<bool>,
     saved_style: Cell<Option<NSWindowStyleMask>>,
+    background_color: RefCell<Retained<NSColor>>,
 }
 
 declare_class!(
@@ -689,6 +690,7 @@ impl WindowDelegate {
             standard_frame: Cell::new(None),
             is_simple_fullscreen: Cell::new(false),
             saved_style: Cell::new(None),
+            background_color: unsafe { NSColor::blackColor().into() },
         });
         let delegate: Retained<WindowDelegate> =
             unsafe { msg_send_id![super(delegate), init] };
@@ -834,10 +836,9 @@ impl WindowDelegate {
                     .setBackgroundColor(Some(&NSColor::clearColor()));
             }
         } else {
-            unsafe {
-                self.window()
-                    .setBackgroundColor(Some(&NSColor::blackColor()));
-            }
+            self.window()
+                .setBackgroundColor(Some(&self.ivars().background_color.borrow()));
+            // .setBackgroundColor(Some(&NSColor::blackColor()));
         }
     }
 
@@ -1764,6 +1765,15 @@ impl WindowExtMacOS for WindowDelegate {
         self.window().hasShadow()
     }
 
+    #[inline]
+    fn set_background_color(&self, r: f64, g: f64, b: f64, a: f64) {
+        let color_value =
+            unsafe { NSColor::colorWithSRGBRed_green_blue_alpha(r, g, b, a) };
+        let mut background_color = self.ivars().background_color.borrow_mut();
+        *background_color = color_value.clone();
+        self.window().setBackgroundColor(Some(&color_value));
+    }
+
     #[inline]
     fn set_has_shadow(&self, has_shadow: bool) {
         self.window().setHasShadow(has_shadow)
