@@ -1,15 +1,17 @@
 use super::cache::ImageCache;
 use super::{AddImage, ImageData, ImageId, ImageLocation};
+use crate::font::FontLibrary;
 use crate::font_introspector::scale::{
     image::{Content, Image as GlyphImage},
     *,
 };
 use crate::font_introspector::zeno::Format;
-use crate::font_introspector::FontRef;
 use core::borrow::Borrow;
 use core::hash::{Hash, Hasher};
 use rustc_hash::FxHashMap;
 
+// const IS_MACOS: bool = cfg!(target_os = "macos");
+
 const SOURCES: &[Source] = &[
     Source::ColorOutline(0),
     Source::ColorBitmap(StrikeWith::BestFit),
@@ -38,28 +40,23 @@ impl GlyphCache {
     pub fn session<'a>(
         &'a mut self,
         images: &'a mut ImageCache,
-        font: FontRef<'a>,
+        font: usize,
+        font_library: &'a FontLibrary,
         coords: &[i16],
         size: f32,
     ) -> GlyphCacheSession<'a> {
         // let quant_size = (size * 32.) as u16;
         let quant_size = size as u16;
-        let entry = get_entry(&mut self.fonts, font.key.value(), coords);
-        let scaler = self
-            .scx
-            .builder(font)
-            // .hint(!IS_MACOS)
-            .hint(true)
-            .size(size)
-            // .normalized_coords(coords)
-            .build();
+        let entry = get_entry(&mut self.fonts, font, coords);
         GlyphCacheSession {
+            font,
             entry,
             images,
-            scaler,
+            font_library,
             max_height: &self.max_height,
             scaled_image: &mut self.img,
             quant_size,
+            scale_context: &mut self.scx,
         }
     }
 
@@ -80,7 +77,7 @@ impl GlyphCache {
 
 fn get_entry<'a>(
     fonts: &'a mut FxHashMap<FontKey, FontEntry>,
-    id: u64,
+    id: usize,
     coords: &[i16],
 ) -> &'a mut FontEntry {
     let key = (id, Coords::Ref(coords));
@@ -100,8 +97,10 @@ fn get_entry<'a>(
 pub struct GlyphCacheSession<'a> {
     entry: &'a mut FontEntry,
     images: &'a mut ImageCache,
-    scaler: Scaler<'a>,
     scaled_image: &'a mut GlyphImage,
+    font: usize,
+    font_library: &'a FontLibrary,
+    scale_context: &'a mut ScaleContext,
     quant_size: u16,
     #[allow(unused)]
     max_height: &'a u16,
@@ -125,65 +124,83 @@ impl<'a> GlyphCacheSession<'a> {
         }
 
         self.scaled_image.data.clear();
-        // let embolden = if IS_MACOS { 0.25 } else { 0. };
-        if Render::new(SOURCES)
-            .format(Format::CustomSubpixel([0.3, 0., -0.3]))
-            // .format(Format::Alpha)
-            // .offset(Vector::new(subpx[0].to_f32(), subpx[1].to_f32()))
-            // .embolden(embolden)
-            // .transform(if cache_key.flags.contains(CacheKeyFlags::FAKE_ITALIC) {
-            //     Some(Transform::skew(
-            //         Angle::from_degrees(14.0),
-            //         Angle::from_degrees(0.0),
-            //     ))
-            // } else {
-            //     None
-            // })
-            .render_into(&mut self.scaler, id, self.scaled_image)
-        {
-            let p = self.scaled_image.placement;
-            let w = p.width as u16;
-            let h = p.height as u16;
-            let req = AddImage {
-                width: w,
-                height: h,
-                has_alpha: true,
-                data: ImageData::Borrowed(&self.scaled_image.data),
-            };
-            let image = self.images.allocate(req)?;
+        let mut font_library_data = self.font_library.inner.lock();
 
-            // let mut top = p.top;
-            // let mut height = h;
+        if let Some(data) = font_library_data.get_data(&self.font) {
+            let mut scaler = self
+                .scale_context
+                .builder(data)
+                // With the advent of high-DPI displays (displays with >300 pixels per inch),
+                // font hinting has become less relevant, as aliasing effects become
+                // un-noticeable to the human eye.
+                // As a result Apple's Quartz text renderer, which is targeted for Retina displays,
+                // now ignores font hint information completely.
+                // .hint(!IS_MACOS)
+                .hint(true)
+                .size(self.quant_size.into())
+                // .normalized_coords(coords)
+                .build();
 
-            // If dimension is None it means that we are running
-            // for the first time and in this case, we will obtain
-            // what the next glyph entries should respect in terms of
-            // top and height values
-            //
-            // e.g: Placement { left: 11, top: 42, width: 8, height: 50 }
-            //
-            // The calculation is made based on max_height
-            // If the rect max height is 50 and the glyph height is 68
-            // and 48 top, then (68 - 50 = 18) height as difference and
-            // apply it to the top (bigger the top == up ^).
-            // if self.max_height > &0 && &h > self.max_height {
-            //     let difference = h - self.max_height;
+            // let embolden = if IS_MACOS { 0.25 } else { 0. };
+            if Render::new(SOURCES)
+                .format(Format::CustomSubpixel([0.3, 0., -0.3]))
+                // .format(Format::Alpha)
+                // .offset(Vector::new(subpx[0].to_f32(), subpx[1].to_f32()))
+                // .embolden(embolden)
+                // .transform(if cache_key.flags.contains(CacheKeyFlags::FAKE_ITALIC) {
+                //     Some(Transform::skew(
+                //         Angle::from_degrees(14.0),
+                //         Angle::from_degrees(0.0),
+                //     ))
+                // } else {
+                //     None
+                // })
+                .render_into(&mut scaler, id, self.scaled_image)
+            {
+                let p = self.scaled_image.placement;
+                let w = p.width as u16;
+                let h = p.height as u16;
+                let req = AddImage {
+                    width: w,
+                    height: h,
+                    has_alpha: true,
+                    data: ImageData::Borrowed(&self.scaled_image.data),
+                };
+                let image = self.images.allocate(req)?;
 
-            //     top -= difference as i32;
-            //     height = *self.max_height;
-            // }
+                // let mut top = p.top;
+                // let mut height = h;
 
-            let entry = GlyphEntry {
-                left: p.left,
-                top: p.top,
-                width: w,
-                height: h,
-                image,
-                is_bitmap: self.scaled_image.content == Content::Color,
-            };
+                // If dimension is None it means that we are running
+                // for the first time and in this case, we will obtain
+                // what the next glyph entries should respect in terms of
+                // top and height values
+                //
+                // e.g: Placement { left: 11, top: 42, width: 8, height: 50 }
+                //
+                // The calculation is made based on max_height
+                // If the rect max height is 50 and the glyph height is 68
+                // and 48 top, then (68 - 50 = 18) height as difference and
+                // apply it to the top (bigger the top == up ^).
+                // if self.max_height > &0 && &h > self.max_height {
+                //     let difference = h - self.max_height;
 
-            self.entry.glyphs.insert(key, entry);
-            return Some(entry);
+                //     top -= difference as i32;
+                //     height = *self.max_height;
+                // }
+
+                let entry = GlyphEntry {
+                    left: p.left,
+                    top: p.top,
+                    width: w,
+                    height: h,
+                    image,
+                    is_bitmap: self.scaled_image.content == Content::Color,
+                };
+
+                self.entry.glyphs.insert(key, entry);
+                return Some(entry);
+            }
         }
 
         None
@@ -192,11 +209,11 @@ impl<'a> GlyphCacheSession<'a> {
 
 #[derive(Clone, PartialEq, Eq, Hash, Debug)]
 struct FontKey {
-    key: (u64, Coords<'static>),
+    key: (usize, Coords<'static>),
 }
 
-impl<'a> Borrow<(u64, Coords<'a>)> for FontKey {
-    fn borrow(&self) -> &(u64, Coords<'a>) {
+impl<'a> Borrow<(usize, Coords<'a>)> for FontKey {
+    fn borrow(&self) -> &(usize, Coords<'a>) {
         &self.key
     }
 }
