@@ -19,6 +19,5 @@ pub use crate::sugarloaf::{
 };
 pub use components::rect::Rect;
 pub use layout::{
-    Content, ContentBuilder, FragmentStyle, FragmentStyleDecoration, UnderlineInfo,
-    UnderlineShape,
+    Content, FragmentStyle, FragmentStyleDecoration, UnderlineInfo, UnderlineShape,
 };
